@@ -29,12 +29,12 @@ plot(1:numiter,fit.sqp$progress$objective,type = "b",
      ylab = "objective",xaxp = c(1,numiter,numiter - 1))
 
 ## ----fit-model-kwdual-small----------------------------------------------
-fit.ip <- mixkwdual(L)
+#  fit.ip <- mixkwdual(L)
 
 ## ----compare-solutions-smaller-------------------------------------------
-cat(sprintf("Objective at SQP solution: %0.16f\n",fit.sqp$value,digits = 16))
-cat(sprintf("Objective at IP solution:  %0.16f\n",fit.ip$value,digits = 16))
-cat(sprintf("Difference in objectives:  %0.4e\n",fit.sqp$value - fit.ip$value))
+#  cat(sprintf("Objective at SQP solution: %0.16f\n",fit.sqp$value,digits = 16))
+#  cat(sprintf("Objective at IP solution:  %0.16f\n",fit.ip$value,digits = 16))
+#  cat(sprintf("Difference in objectives:  %0.4e\n",fit.sqp$value - fit.ip$value))
 
 ## ----sim-data-large, eval=TRUE-------------------------------------------
 L <- simulatemixdata(1e5,100)$L
@@ -45,13 +45,13 @@ timing <- system.time(fit.sqp <- mixsqp(L))
 cat(sprintf("Computation took %0.2f seconds\n",timing["elapsed"]))
 
 ## ----fit-model-ip-large--------------------------------------------------
-timing <- system.time(fit.ip  <- mixkwdual(L))
-cat(sprintf("Computation took %0.2f seconds\n",timing["elapsed"]))
+#  timing <- system.time(fit.ip  <- mixkwdual(L))
+#  cat(sprintf("Computation took %0.2f seconds\n",timing["elapsed"]))
 
 ## ----compare-solutions-large---------------------------------------------
-cat(sprintf("Objective at SQP solution: %0.16f\n",fit.sqp$value,digits = 16))
-cat(sprintf("Objective at IP solution:  %0.16f\n",fit.ip$value,digits = 16))
-cat(sprintf("Difference in objectives:  %0.4e\n",fit.sqp$value - fit.ip$value))
+#  cat(sprintf("Objective at SQP solution: %0.16f\n",fit.sqp$value,digits = 16))
+#  cat(sprintf("Objective at IP solution:  %0.16f\n",fit.ip$value,digits = 16))
+#  cat(sprintf("Difference in objectives:  %0.4e\n",fit.sqp$value - fit.ip$value))
 
 ## ----session-info, eval=TRUE---------------------------------------------
 sessionInfo()
